# Copyright 2014 Bevbot LLC, All Rights Reserved
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from optparse import make_option
import sys

from pykeg import EPOCH
from pykeg.core.util import get_version

from django.core.management.base import BaseCommand
from django.db import connection

from django.contrib.staticfiles.management.commands import collectstatic
from south.management.commands import migrate
from south.management.commands import syncdb
from pykeg.core.management.commands import kb_regen_stats

from pykeg.core import models
from pykeg.core import checkin


def run(cmd, args=[]):
    cmdname = cmd.__module__.split('.')[-1]
    arg_str = ' '.join('%s' % a for a in args)
    print '--- Running command: %s %s' % (cmdname, arg_str)
    cmd.run_from_argv([sys.argv[0], cmdname] + args)


class Command(BaseCommand):
    help = u'Perform post-upgrade tasks.'
    option_list = BaseCommand.option_list + (
        make_option('--skip_static', action='store_true', dest='skip_static', default=False,
            help='Skip `kegbot collectstatic` during upgrade. (Not recommended.)'),
        make_option('--skip_stats', action='store_true', dest='skip_stats', default=False,
            help='Skip `kegbot kb_regen_stats` during upgrade. (Not recommended.)'),
    )

    def handle(self, *args, **options):
        self.do_epoch_upgrades()
        run(syncdb.Command(), args=['--noinput', '-v', '0'])
        run(migrate.Command(), args=['-v', '0'])

        if not options.get('skip_stats'):
            run(kb_regen_stats.Command())

        if not options.get('skip_static'):
            run(collectstatic.Command(), args=['--noinput'])

        site = models.KegbotSite.get()
        site.epoch = EPOCH
        site.server_version = get_version()
        site.save()

        # Refresh any news (since we have a new version).
        try:
            checkin.checkin(timeout=5.0, quiet=True)
        except (checkin.CheckinError, Exception):
            pass

        print ''
        print 'Upgrade complete!'

    def get_epoch(self):
        cursor = connection.cursor()
        cursor.execute("SELECT epoch FROM core_kegbotsite")
        row = cursor.fetchone()
        if not row:
            return None
        return int(row[0])

    def do_epoch_upgrades(self):
        installed = self.get_epoch()
        print '--- Current epoch: %s' % installed
        if installed == EPOCH or not installed:
            return

        print 'Performing epoch upgrades ...'
        for version in range(installed + 1, EPOCH + 1):
            fn = getattr(self, 'to_%s' % version, None)
            if fn:
                print '  ~ %s' % version
                fn()
            else:
                print '  ~ %s (no-op)' % version

    def to_101(self):
        cursor = connection.cursor()
        cursor.execute('DELETE FROM south_migrationhistory WHERE app_name = %s', ['twitter'])

    def to_102(self):
        cursor = connection.cursor()
        cursor.execute('DELETE FROM south_migrationhistory WHERE app_name = %s', ['foursquare'])

    def to_103(self):
        cursor = connection.cursor()
        cursor.execute('DELETE FROM south_migrationhistory WHERE app_name = %s', ['untappd'])

    def to_104(self):
        cursor = connection.cursor()
        cursor.execute('DROP TABLE IF EXISTS djkombu_message')
        cursor.execute('DROP TABLE IF EXISTS djkombu_queue')



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'NotificationSettings'
        db.create_table(u'notification_notificationsettings', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('backend', self.gf('django.db.models.fields.TextField')()),
            ('keg_tapped', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('session_started', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('keg_volume_low', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('keg_ended', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'notification', ['NotificationSettings'])


    def backwards(self, orm):

        # Deleting model 'NotificationSettings'
        db.delete_table(u'notification_notificationsettings')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'notification.notificationsettings': {
            'backend': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg_ended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keg_tapped': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'keg_volume_low': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_started': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['notification']


# Copyright 2014 Bevbot LLC, All Rights Reserved
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError
from django.db import transaction

from pykeg.core import models


class Command(BaseCommand):
    args = '<from> <to>'
    help = 'Renames user from <from> to <to>.'

    def handle(self, *args, **options):
        if len(args) < 2:
            raise CommandError('Must specify <from> and <to>')

        from_username = args[0]
        to_username = args[1]

        if from_username == 'guest':
            raise CommandError('Cannot rename the guest user.')

        with transaction.atomic():
            try:
                user = models.User.objects.get(username=from_username)
            except models.User.DoesNotExist:
                raise CommandError('User named "{}" does not exist'.format(from_username))

            user.username = to_username
            user.save()

        print '"{}" has been renamed "{}"'.format(from_username, to_username)


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

import json

class Migration(DataMigration):

    def forwards(self, orm):
        for row in orm['plugin.PluginData'].objects.all():
            try:
                # Do nothing if already valid json.
                json.loads(row.value)
            except ValueError:
                row.value = json.dumps(row.value)
                row.save()

    def backwards(self, orm):
        pass

    models = {
        u'plugin.plugindata': {
            'Meta': {'unique_together': "(('plugin_name', 'key'),)", 'object_name': 'PluginData'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '127'}),
            'plugin_name': ('django.db.models.fields.CharField', [], {'max_length': '127'}),
            'value': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['plugin']
    symmetrical = True



# Copyright 2014 Bevbot LLC, All Rights Reserved
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import pytz

from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import Library
from django.template import Node
from django.template import VariableDoesNotExist
from django.template import TemplateSyntaxError
from django.template import Variable
from django.template.defaultfilters import pluralize
from django.utils import timezone
from django.utils.safestring import mark_safe

from kegbot.util import kbjson
from kegbot.util import units
from kegbot.util import util

from pykeg.core import models
from pykeg.web.charts import charts

register = Library()


@register.inclusion_tag('kegweb/mugshot_box.html', takes_context=True)
def mugshot_box(context, user, boxsize=0):
    return {
        'user': user,
        'boxsize': boxsize,
        'guest_info': context.get('guest_info', None),
        'STATIC_URL': context.get('STATIC_URL')
    }


@register.inclusion_tag('kegweb/picture-gallery.html')
def gallery(picture_or_pictures, thumb_size='span2', gallery_id=''):
    c = {}
    if not hasattr(picture_or_pictures, '__iter__'):
        c['gallery_pictures'] = [picture_or_pictures]
    else:
        c['gallery_pictures'] = picture_or_pictures
    c['thumb_size'] = thumb_size
    c['gallery_id'] = gallery_id
    return c


@register.inclusion_tag('kegweb/badge.html')
def badge(amount, caption, style='', is_volume=False, do_pluralize=False):
    if is_volume:
        amount = mark_safe(VolumeNode.format(amount, 'mL'))
    if do_pluralize:
        caption += pluralize(amount)
    return {
        'badge_amount': amount,
        'badge_caption': caption,
        'badge_style': style,
    }


@register.inclusion_tag('kegweb/includes/progress_bar.html')
def progress_bar(progress_int, extra_css=''):
    c = {}
    try:
        progress_int = max(int(progress_int), 0)
    except ValueError:
        progress_int = 0
    progress_int = min(progress_int, 100)

    c['progress_int'] = progress_int
    c['extra_css'] = extra_css
    if progress_int < 10:
        bar_type = 'bar-danger'
    elif progress_int < 25:
        bar_type = 'bar-warning'
    else:
        bar_type = 'bar-success'
    c['bar_type'] = bar_type
    return c


### navitem

@register.tag('navitem')
def navitem(parser, token):
    """{% navitem <viewname> <title> [exact] %}"""
    tokens = token.split_contents()
    if len(tokens) < 3:
        raise TemplateSyntaxError('%s requires at least 3 tokens' % tokens[0])
    return NavitemNode(*tokens[1:])


class NavitemNode(Node):
    def __init__(self, *args):
        self._viewname = args[0]
        self._title = args[1]
        self._exact = 'exact' in args[2:]

    def render(self, context):
        viewname = Variable(self._viewname).resolve(context)
        title = Variable(self._title).resolve(context)
        if viewname.startswith('/'):
            urlbase = viewname
        else:
            urlbase = reverse(viewname)

        request_path = context['request_path']

        if self._exact:
            active = (request_path == urlbase)
        else:
            active = request_path.startswith(urlbase)
        if active:
            res = '<li class="active">'
        else:
            res = '<li>'
        res += '<a href="%s">%s</a></li>' % (urlbase, title)
        return res


### timeago

@register.tag('timeago')
def timeago(parser, token):
    """{% timeago <timestamp> %}"""
    tokens = token.contents.split()
    if len(tokens) != 2:
        raise TemplateSyntaxError('%s requires 2 tokens' % tokens[0])
    return TimeagoNode(tokens[1])


class TimeagoNode(Node):
    def __init__(self, timestamp_varname):
        self._timestamp_varname = timestamp_varname

    def render(self, context):
        tv = Variable(self._timestamp_varname)
        ts = tv.resolve(context)

        # Try to set time zone information.
        if settings.TIME_ZONE and not settings.USE_TZ:
            try:
                tz = pytz.timezone(settings.TIME_ZONE)
                ts = tz.localize(ts)
            except pytz.UnknownTimeZoneError:
                pass

        iso = ts.isoformat()
        alt = timezone.localtime(ts).strftime("%A, %B %d, %Y %I:%M%p")
        return '<abbr class="timeago" title="%s">%s</abbr>' % (iso, alt)


### temperature

@register.tag('temperature')
def temperature_tag(parser, token):
    """{% temperature <temp_c> %}"""
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise TemplateSyntaxError('%s requires at least 2 tokens' % tokens[0])
    return TemperatureNode(tokens[1])


class TemperatureNode(Node):
    TEMPLATE = "%(amount)s&deg; %(unit)s"

    def __init__(self, varname):
        self.varname = varname

    def render(self, context):
        v = Variable(self.varname)
        try:
            amount = v.resolve(context)
        except (VariableDoesNotExist, ValueError):
            raise
            amount = 'unknown'

        unit = 'C'
        kbsite = models.KegbotSite.get()
        if kbsite.temperature_display_units == 'f':
            unit = 'F'
            amount = util.CtoF(amount)

        return self.TEMPLATE % {'amount': amount, 'unit': unit}

### volume


@register.tag('volume')
def volumetag(parser, token):
    """{% volume <amount> %}"""
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise TemplateSyntaxError('%s requires at least 2 tokens' % tokens[0])
    return VolumeNode(tokens[1], tokens[2:])


class VolumeNode(Node):
    TEMPLATE = """
      <span class="hmeasure %(extra_css)s" title="%(title)s">
        <span class="num">%(amount)s</span>
        <span class="unit">%(units)s</span>
      </span>""".strip()

    def __init__(self, volume_varname, extra_args):
        self._volume_varname = volume_varname
        self._extra_args = extra_args

    def render(self, context):
        tv = Variable(self._volume_varname)
        try:
            num = float(tv.resolve(context))
        except (VariableDoesNotExist, ValueError):
            num = 'unknown'
        unit = 'mL'
        make_badge = 'badge' in self._extra_args
        return self.format(num, unit, make_badge)

    @classmethod
    def format(cls, amount, units, make_badge=False):
        if amount < 0:
            amount = 0
        ctx = {
            'units': units,
            'amount': amount,
            'title': '%s %s' % (amount, units),
            'extra_css': 'badge ' if make_badge else '',
        }
        return cls.TEMPLATE % ctx

### drinker


@register.tag('drinker_name')
def drinker_name_tag(parser, token):
    """{% drinker_name <drink_or_user_obj> [nolink] %}"""
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise TemplateSyntaxError('%s requires at least 2 tokens' % tokens[0])
    return DrinkerNameNode(tokens[1], tokens[2:])


class DrinkerNameNode(Node):
    def __init__(self, drink_varname, extra_args):
        self._varname = drink_varname
        self._extra_args = extra_args

    def render(self, context):
        obj = Variable(self._varname)
        try:
            obj = obj.resolve(context)
        except (VariableDoesNotExist, ValueError):
            obj = None

        user = None
        if obj:
            if isinstance(obj, models.Drink) or isinstance(obj, models.SystemEvent):
                user = obj.user
            elif isinstance(obj, models.User):
                user = obj
        if user:
            if 'nolink' in self._extra_args:
                return user.username
            else:
                return '<a href="%s">%s</a>' % (reverse('kb-drinker', args=[user.username]), user.username)
        return context['guest_info']['name']


### chart

@register.tag('chart')
def chart(parser, tokens):
    """{% chart <charttype> <obj> width height %}"""
    tokens = tokens.contents.split()
    if len(tokens) < 4:
        raise TemplateSyntaxError('chart requires at least 4 arguments')
    charttype = tokens[1]
    try:
        width = int(tokens[-2])
        height = int(tokens[-1])
    except ValueError:
        raise TemplateSyntaxError('invalid width or height')
    args = tokens[2:-2]
    return ChartNode(charttype, width, height, args)


class ChartNode(Node):
    CHART_TMPL = '''
    <!-- begin chart %(chart_id)s -->
    <div id="chart-%(chart_id)s-container"
        style="height: %(height)spx; width: %(width)spx;"
        class="kb-chartbox"></div>
    <script type="text/javascript">
      var chart_%(chart_id)s;
      $(document).ready(function() {
        var chart_data = %(chart_data)s;
        chart_%(chart_id)s = new Highcharts.Chart(chart_data);
      });
    </script>
    <!-- end chart %(chart_id)s -->

    '''
    ERROR_TMPL = '''
    <!-- begin chart %(chart_id)s -->
    <div id="chart-%(chart_id)s-container"
        style="height: %(height)spx; width: %(width)spx;"
        class="kb-chartbox-error">
      %(error_str)s
    </div>
    <!-- end chart %(chart_id)s -->
    '''

    def __init__(self, charttype, width, height, args):
        self._charttype = charttype
        self._width = width
        self._height = height
        self._args = args

        self._chart_fn = getattr(charts, 'chart_%s' % (self._charttype,), None)

    def _get_chart_id(self, context):
        # TODO(mikey): Is there a better way to store _CHART_ID?
        if not hasattr(context, '_CHART_ID'):
            context._CHART_ID = 0
        context._CHART_ID += 1
        return context._CHART_ID

    def show_error(self, error_str):
        ctx = {
            'error_str': error_str,
            'chart_id': 0,
            'width': self._width,
            'height': self._height,
        }
        return ChartNode.ERROR_TMPL % ctx

    def render(self, context):
        if not self._chart_fn:
            return self.show_error("Unknown chart type: %s" % self._charttype)

        chart_id = self._get_chart_id(context)
        obj = Variable(self._args[0]).resolve(context)

        metric_volumes = context.get('metric_volumes', False)
        temperature_units = context.get('temperature_display_units', 'f')

        try:
            chart_result = self._chart_fn(obj, metric_volumes=metric_volumes,
                temperature_units=temperature_units)
        except charts.ChartError, e:
            return self.show_error(str(e))

        chart_base = {
            'chart': {
                'borderColor': '#eeeeff',
                'borderWidth': 0,
                'renderTo': 'chart-%s-container' % chart_id,
            },
            'credits': {
                'enabled': False,
            },
            'legend': {
                'enabled': False,
            },
            'margin': [0, 0, 0, 0],
            'title': {
                'text': None,
            },
            'yAxis': {
                'labels': {
                    'align': 'left'
                },
                'title': {
                    'text': None,
                }
            },
        }

        chart_data = chart_base
        for k, v in chart_result.iteritems():
            if k not in chart_data:
                chart_data[k] = v
            elif isinstance(v, dict):
                chart_data[k].update(v)
            else:
                chart_data[k] = v
        chart_data = kbjson.dumps(chart_data, indent=None)

        ctx = {
            'chart_id': chart_id,
            'width': self._width,
            'height': self._height,
            'chart_data': chart_data,
        }

        return ChartNode.CHART_TMPL % ctx


@register.filter
def volume(text, fmt='pints'):
    try:
        vol = units.Quantity(float(text))
    except ValueError:
        return text
    if fmt == 'pints':
        res = vol.InPints()
    elif fmt == 'liters':
        res = vol.InLiters()
    elif fmt == 'ounces':
        res = vol.InOunces()
    elif fmt == 'gallons':
        res = vol.InUSGallons()
    elif fmt == 'twelveounces':
        res = vol.InTwelveOunceBeers()
    elif fmt == 'halfbarrels':
        res = vol.InHalfBarrelKegs()
    else:
        raise TemplateSyntaxError('Unknown volume format: %s' % fmt)
    return float(res)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PluginData.value'
        db.alter_column(u'plugin_plugindata', 'value', self.gf('pykeg.core.jsonfield.JSONField')())

    def backwards(self, orm):

        # Changing field 'PluginData.value'
        db.alter_column(u'plugin_plugindata', 'value', self.gf('django.db.models.fields.TextField')())

    models = {
        u'plugin.plugindata': {
            'Meta': {'unique_together': "(('plugin_name', 'key'),)", 'object_name': 'PluginData'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '127'}),
            'plugin_name': ('django.db.models.fields.CharField', [], {'max_length': '127'}),
            'value': ('pykeg.core.jsonfield.JSONField', [], {})
        }
    }

    complete_apps = ['plugin']